from kunapi.common import *
from .. import config
import random,urllib,asyncio,websockets,smtplib,datetime,chardet,copy
import kcredis,kccache,kcmysql
import kcsqlite
from kwebs.utill.db import mongodb as kcwsmongodb
from kchttp.httpclass import Http
from kwebs.utill.queues import Queues
from kwebs.utill.db import model
redisobj=kcredis.redis
python_version=platform.python_version()
if python_version[0:3]!='3.8':
    print("\033[1;31;40m "+config.kwebs['name']+"-"+config.kwebs['version']+"依赖python3.8，与你现在的python"+python_version+"不兼容,推荐安装python3.8")
    exit()
class kcwebssocket:
    "websocket服务端"
    __clientlists={} #所有客户端绑定的对象

    __group={} #组
    __clientidgroup={}

    __uid={} #clientid绑定的uid
    __clientiduid={}

    def bindUid(self,clientid,uid):
        """将clientid与uid绑定，以便通过sendToUid(uid)发送数据

        clientid 客户端id

        uid uid与client_id是一对多的关系，系统允许一个uid下有多个client_id
        """
        try:
            self.__uid[uid]
        except KeyError:
            self.__uid[uid]=[]
        if clientid not in self.__uid[uid]:
            self.__uid[uid].append(clientid)
            self.__clientiduid[clientid]=uid
    def unbindUid(self,clientid,uid=0):
        """将clientid与uid解绑 当clientid下线（连接断开）时会自动与uid解绑，开发者无需在onClose事件调用unbindUid
        
        clientid 客户端id

        uid 数字或者字符串 非必传
        """
        if not uid:
            uid=self.getuid(clientid)
        if uid:
            try:
                self.__uid[uid]
            except KeyError:
                pass
            else:
                try:
                    self.__uid[uid].remove(clientid)
                except KeyError:
                    pass
                if not self.__uid[uid]:
                    del self.__uid[uid]
            try:
                del self.__clientiduid[clientid]
            except KeyError:
                pass
    def getuid(self,clientid):
        """通过clientid查询uid
        
        clientid 客户端id

        return uid
        """
        try:
            return self.__clientiduid[clientid]
        except KeyError:
            return 0
    def joinGroup(self,clientid,group):
        """将clientid加入某个组
        
        clientid 客户端id

        Group 组名 Group与client_id是一对多的关系，系统允许一个Group下有多个client_id
        """
        try:
            self.__group[group]
        except KeyError:
            self.__group[group]=[]
        if clientid not in self.__group[group]:
            self.__group[group].append(clientid)
            self.__clientidgroup[clientid]=group
    def getGroup(self,clientid):
        """通过clientid查询组名
        
        clientid 客户端id

        return 组名
        """
        try:
            return self.__clientidgroup[clientid]
        except KeyError:
            return ''
    def leaveGroup(self,clientid,group=''):
        """将clientid从某个组中删除
        
        clientid 客户端id

        group 组名
        """
        if not group:
            group=self.getGroup(clientid)
        if group:
            try:
                self.__group[group]
            except KeyError:
                pass
            else:
                try:
                    self.__group[group].remove(clientid)
                except KeyError:
                    pass
                if not self.__group[group]:
                    self.ungroup(group)
            try:
                del self.__clientidgroup[clientid]
            except KeyError:
                pass
    def ungroup(self,group):
        """解散分组。 解散分组后所有属于这个分组的用户的连接将被移出分组，此分组将不再存在，除非再次调用 joinGroup

        group 组名
        """
        clientidarr=self.__group[group]
        try:
            del self.__group[group]
        except KeyError:
            pass
        for clientid in clientidarr:
            try:
                del self.__clientidgroup[clientid]
            except KeyError:
                pass
    def getClientIdCountByGroup(self,group):
        """获取某分组当前在线成连接数（多少clientid在线）
        
        group 组名

        return int 返回一个数字
        """
        try:
            self.__group[group]
        except KeyError:
            return 0
        else:
            return len(self.__group[group])
    def getAllClientIdCount(self):
        """获取当前在线连接总数（多少client_id在线）
        
        return int 返回一个数字
        """
        return len(self.__clientlists.keys())
    def getGroupCount(self):
        """获取组数量
        
        return int 返回一个数字
        """
        return len(self.__group)
    def getGroupname(self):
        """获取组名称
        
        return list 返回一列表
        """
        return list(self.__group)
    
    async def sendToUid(self,uid,message):
        """向uid绑定的所有在线clientid发送数据

        uid uid可以是字符串、数字、或者包含uid的列表。如果为列表，则是给列表内所有uid发送数据

        message 要发送的数据（字符串类型）
        """
        if isinstance(uid,str):
            for clientid in self.__uid[uid]:
                try:
                    await self.send_client(clientid,message)
                except KeyError:
                    self.unbindUid(clientid,uid)
        elif isinstance(uid,list):
            for uids in uid:
                for clientid in self.__uid[uids]:
                    try:
                        await self.send_client(clientid,message)
                    except KeyError:
                        self.unbindUid(clientid,uids)
    async def sendToGroup(self,group,message,exclude_clientid=[]):
        """向某个分组的所有在线clientid发送数据。

        group 组名

        message 要发送的数据（字符串类型）

        exclude_clientid clientid组成的列表。exclude_clientid列表中指定的clientid将被排除在外，不会收到本次发的消息
        """
        try:
            self.__group[group]
        except KeyError:
            pass
        else:
            i=1
            for client in self.__group[group]:
                i+=1
                if exclude_clientid:
                    if client not in exclude_clientid:
                        try:
                            await self.send_client(client,message)
                        except KeyError:
                            self.leaveGroup(client,group)
                else:
                    try:
                        await self.send_client(client,message)
                    except KeyError:
                        self.leaveGroup(client,group)
    async def send_all(self,message):
        "给所有人发送消息，包括自己"
        for clientid in self.__clientlists.keys():
            await self.send_client(clientid,message)
    async def send_client(self,clientid,message):
        "给所指定客户端发送消息"
        websockets=self.__clientlists[clientid]
        try:
            await websockets.send(message)
        except:pass
    async def onConnect(self,clientid,params):
        "客户端发来连接时"
        if config.app['app_debug']:
            print("连接成功",clientid)
    async def onMessage(self,clientid,recv_text):
        "当客户端发来数据"
        await self.send_client(clientid,recv_text) #给当前用户发送消息
    async def onClose(self,clientid):
        "客户端与websocket的连接断开时触发"
        if config.app['app_debug']:
            print("onClose",clientid)
        await self.CloseSocket(clientid)
    async def CloseSocket(self,clientid):
        "关闭当前客户端socket"
        websockets=self.__clientlists[clientid]
        await websockets.close()
        del self.__clientlists[clientid]
        self.unbindUid(clientid)
        self.leaveGroup(clientid)
    async def __main2(self,clientid,websocket):
        "服务器端主逻辑"
        try:
            async for message in websocket:
                await self.onMessage(clientid, message)
        except:pass
        await self.onClose(clientid)
        await self.CloseSocket(clientid)
    async def __main1(self,clientid,path):
        t = urllib.parse.parse_qs(urllib.parse.urlparse(path).query)
        params={}
        for key in t.keys():
            params[key]=t[key][0]
        await self.onConnect(clientid, params)
    async def __main(self,websocket,path):
        "服务器端主逻辑"
        clientid=md5(str(random.random()))
        self.__clientlists[clientid]=websocket
        task1=asyncio.ensure_future(self.__main1(clientid,path))
        task2=asyncio.ensure_future(self.__main2(clientid,websocket))
        await task1
        await task2

        # task1=asyncio.create_task(self.__main1(clientid,path))
        # task2=asyncio.create_task(self.__main2(clientid,websocket,path))
        # await task1
        # await task2
    def start(self,ip='0.0.0.0',port='39020'):
        "启动websoeket服务"
        asyncio.set_event_loop(asyncio.new_event_loop()) # 防止出现RuntimeError
        asyncio.get_event_loop().run_until_complete(websockets.serve(self.__main,ip,port))
        asyncio.get_event_loop().run_forever()
# def start():
#     kwebsocket=kcwebssocket()
#     kwebsocket.start()
def before_request():pass

def send_mail(user,text="邮件内容",theme="邮件主题",recNick="收件人昵称"):
    """发送邮件

    参数 user：接收邮件的邮箱地址

    参数 text：邮件内容

    参数 theme：邮件主题

    参数 recNick：收件人昵称

    return Boolean类型
    """
    from email.mime.text import MIMEText
    from email.utils import formataddr

    ret=True
    if not theme:
        theme=config.email['theme']
    if not recNick:
        recNick=config.email['recNick']
    try:
        msg=MIMEText(text,'plain','utf-8')
        msg['From']=formataddr([config.email['sendNick'],config.email['sender']]) 
        msg['To']=formataddr([recNick,user]) 
        msg['Subject']=theme

        server=smtplib.SMTP_SSL("smtp.qq.com", 465) 
        server.login(config.email['sender'], config.email['pwd']) 
        server.sendmail(config.email['sender'],[user,],msg.as_string())
        server.quit()
    except Exception:
        ret=False
    return ret
def Template(path,**context):
    "模板渲染引擎函数,使用配置的模板路径"
    return Templates(path,**context)
def Templates(path,**context):
    "模板渲染引擎函数，需要完整的模板目录文件"
    from mako.template import Template as kcwsTemplate
    from mako.lookup import TemplateLookup
    lookup = TemplateLookup(directories=[''])
    if (path[:1]=="/" or path[1:2]==":") and ('.html' in path or '.shtml' in path or '.htm' in path):
        t=kcwsTemplate(filename=path,module_directory=config.cache['path']+"/Template",lookup=lookup)
    else:
        t=lookup.get_template(path)
    body=t.render(**context)
    return body
def kcwsTemp(contents,**context):
    "模板渲染引擎函数，传字符串进来"
    from mako.template import Template as kcwsTemplate
    from mako.lookup import TemplateLookup
    lookup = TemplateLookup(directories=[''])
    t=kcwsTemplate(contents,lookup=lookup,module_directory=config.cache['path']+"/Template")
    body=t.render(**context)
    return body
def getfunction(strs,reload=False):
    """获取指定文件对象
    
    strs :app.index.common.autoload  获取app/index/common/目录下的autoload对象

    reload 是否重新加载已导入的模块（是否每次加载修改后的模块）
    """
    obj=importlib.import_module(strs)
    if reload:
        importlib.reload(obj)
    return obj
mysqldbobj=kcmysql.mysql
def mysql(table=None,configss=None):
    """mysql数据库操作实例
    
    参数 table：表名

    参数 configss 数据库配置  可以传数据库名字符串
    """
    global mysqldbobj
    if not mysqldbobj:
        mysqldbobj=kcmysql.mysql
    if table is None:
        return mysqldbobj
    elif configss:
        return mysqldbobj.connect(configss).table(table)
    else:
        return mysqldbobj.connect(config.database).table(table)
def sqlite(table=None,configss=None):
    """sqlite数据库操作实例
    
    参数 table：表名

    参数 configss 数据库配置  可以传数据库名字符串
    """
    
    dbs=kcsqlite.sqlite #kcwssqlite.sqlite()
    if table is None:
        return dbs
    elif configss:
        return dbs.connect(configss).table(table)
    else:
        return dbs.connect(config.sqlite).table(table)
def mongo(table=None,configss=None):
    """mongodb数据库操作实例
    
    参数 table：表名(mongodb数据库集合名)

    参数 configss mongodb数据库配置  可以传数据库名字符串
    """
    mObj=kcwsmongodb.mongo()
    if table is None:
        return mObj
    elif configss:
        return mObj.connect(configss).table(table)
    else:
        return mObj.connect(config.mongo).table(table)
def M(table=None,confi=None):
    """数据库操作实例
    
    参数 table：表名

    参数 confi 数据库配置  可以传数据库名字符串
    """
    if confi:
        if confi['type']=='sqlite':
            return sqlite(table,confi)
        else:
            return mysql(table,confi)
    else:
        if config.database['type']=='sqlite':
            return sqlite(table)
        else:
            return mysql(table)
def set_cache(name,values,expire="no"):
    """设置缓存

    参数 name：缓存名

    参数 values：缓存值

    参数 expire：缓存有效期 0表示永久  单位 秒
    
    return Boolean类型
    """
    return kccache.cache.set_config(config=config.cache).set_cache(name,values,expire)
def get_cache(name):
    """获取缓存

    参数 name：缓存名

    return 或者的值
    """
    return kccache.cache.set_config(config=config.cache).get_cache(name)
def del_cache(name):
    """删除缓存

    参数 name：缓存名

    return Boolean类型
    """
    return kccache.cache.set_config(config=config.cache).del_cache(name)
def dateoperator(date,years=0,formats='%Y%m%d%H%M%S',months=0, days=0, hours=0, minutes=0,seconds=0,
                 leapdays=0, weeks=0, microseconds=0,
                 year=None, month=None, day=None, weekday=None,
                 yearday=None, nlyearday=None,
                 hour=None, minute=None, second=None, microsecond=None):
    """日期相加减计算
    date 2019-10-10
    formats 设置需要返回的时间格式 默认%Y%m%d%H%M%S
    
    years 大于0表示加年  反之减年
    months 大于0表示加月  反之减月
    days 大于0表示加日  反之减日

    return %Y%m%d%H%M%S
    """
    from dateutil.relativedelta import relativedelta
    formatss='%Y%m%d%H%M%S'
    date=re.sub('[-年/月:：日 时分秒]','',date)
    if len(date) < 8:
        return None
    if len(date) < 14:
        s=14-len(date)
        i=0
        while i < s:
            date=date+"0"
            i=i+1
    d = datetime.datetime.strptime(date, formatss)
    strs=(d + relativedelta(years=years,months=months, days=days, hours=hours, minutes=minutes,seconds=seconds,
                 leapdays=leapdays, weeks=weeks, microseconds=microseconds,
                 year=year, month=month, day=day, weekday=weekday,
                 yearday=yearday, nlyearday=nlyearday,
                 hour=hour, minute=minute, second=second, microsecond=microsecond))
    strs=strs.strftime(formats)
    return strs
def get_kwebs_folder():
    '获取当前框架目录'
    return os.path.split(os.path.realpath(__file__))[0][:-7] #当前框架目录
if not os.path.exists(get_kwebs_folder()+"/pid/"):
    os.makedirs(get_kwebs_folder()+"/pid/", exist_ok=True)
def kill_all_kwebs_pid(types='all'):
    """结束kwebs框架的所有子进程
    
    types 可选 all 和 kwebs
    """
    if types=='all':
        kill_all_pid()
    folder=get_kwebs_folder()+"/pid"
    lis=os.listdir(folder)
    for files in lis:
        if os.path.isfile(folder+"/"+files):
            f=open(folder+"/"+files,'r')
            pid=f.read()
            f.close()
            kill_pid(pid)
            os.remove(folder+"/"+files)

def list_to_tree(data, pk = 'id', pid = 'pid', child = 'lowerlist', root=0,childstatus=True):
    """列表转换tree
    
    data 要转换的列表

    pk 关联节点字段

    pid 父节点字段

    lowerlist 子节点列表

    root 主节点值

    childstatus 当子节点列表为空时是否需要显示子节点字段
    """
    arr = []
    for v in data:
        if v[pid] == root:
            kkkk=list_to_tree(data,pk,pid,child,v[pk],childstatus)
            if childstatus:
                v[child]=kkkk
            else:
                if kkkk:
                    v[child]=kkkk
            arr.append(v)
    return arr
def file_set_content(filename,data,encoding="utf-8"):
    """写入文件内容
    
    filename 完整文件名

    data 要写入的内容

    encoding 保存编码
    """
    f=open(filename,'w',encoding=encoding)
    f.write(data)
    f.close()
    return True
def file_get_content(filename,cur_encoding='utf-8',encoding=False):
    """获取文件内容
    
    filename 完整文件名

    cur_encoding 指定编码获取文件内容

    encoding 是否返回文件编码  默认否
    """
    fileData=''
    if os.path.isfile(filename):
        if encoding:
            with open(filename, 'rb') as f:
                cur_encoding = chardet.detect(f.read())['encoding']
                # print('cur_encoding',cur_encoding)
        #用获取的编码读取该文件而不是python3默认的utf-8读取。
        with open(filename,encoding=cur_encoding) as file:
            fileData = file.read()
    if encoding:
        return fileData,cur_encoding
    else:
        return fileData
class kwebssign:
    def getsign(params):
        "获取签名"
        if is_index(params,'sign'):
            del params['sign']
        content=kwebssign.getSignContent(params)
        return md5(content)
    def exsignpra(params):
        "生成签名参数"
        params['time']=times()
        params['rands']=randoms()
        params['sign']=kwebssign.getsign(params)
        return params
    def getSignContent(params):
        "字典排序"
        param={}
        for i in sorted (params) : 
            param[i]=params[i]
        i=0
        strs=""
        for k in param:
            if k:
                if isinstance(k,dict):
                    k=json_encode(k)
                    k=k.replace('"', '')
                    k=k.replace("'", '')
                if param[k]:
                    if i==0:
                        strs+=str(k)+"="+str(param[k])
                    else:
                        strs+="&"+str(k)+"="+str(param[k])
            i+=1
        return strs
kcwebszip=kcwszip
kwebstar=kcwstar
class response:
    tpldata={} #公共模板变量
    def tpl(path=None,status='200 ok',response_cache=False,ETag=None,header={"Content-Type":"text/html; charset=utf-8"},**context):
        """返回模板内容
        
        path 文件地址

        status 响应状态码

        response_cache 是否启用浏览器缓存  响应状态码200 ok时有效

        ETag 缓存标识  响应状态码200 ok时有效

        header 响应头
        """
        for k in dir(response):
            if k not in ['download','json','pic','redirect','tpl','tpldata','video'] and k[-2:]!='__':
                try:
                    context[k]=response.__dict__[k]
                except KeyError:
                    pass
        context['config']=config
        headers=copy.deepcopy(header)
        getroutecomponent=globals.VAR.component
        if path:
            if (path[:1]=="/" or path[1:2]==":") and ('.' in path[-8:]):
                Temppath=path
            elif (path[:1]=="/"):
                Temppath=config.app['tpl_folder']+path+".html"
            else:
                Temppath=config.app['tpl_folder']+"/"+getroutecomponent[1]+"/controller/"+getroutecomponent[2]+"/tpl/"+path+".html"
        else:
            Temppath=config.app['tpl_folder']+"/"+getroutecomponent[1]+"/controller/"+getroutecomponent[2]+"/tpl/"+getroutecomponent[3]+"/"+getroutecomponent[4]+".html"
        # print('Temppath',Temppath)
        if status=='200 ok' and response_cache:
            if not ETag:
                ttt=''
                for k in context.keys():
                    ttt+=k+str(context[k])
                ETag=md5(Temppath+ttt+globals.HEADER.URL)
            try:
                HTTP_IF_NONE_MATCH=globals.HEADER.GET['HTTP_IF_NONE_MATCH']
            except:
                HTTP_IF_NONE_MATCH=None
            if HTTP_IF_NONE_MATCH and HTTP_IF_NONE_MATCH==ETag:
                status="304 Not Modified"
                body=''
            else:
                # if isinstance(response_cache,int) and response_cache>1:
                #     headers['response_cache']=str(response_cache)+" s"
                #     set_cache(ETag,1,response_cache)
                # else:
                #     headers['response_cache']="default"
                #     set_cache(ETag,1)
                body=Template(Temppath,**context)
            dateArray = datetime.datetime.utcfromtimestamp(times()-86400)
            otherStyleTime = dateArray.strftime('%a, %d %b %Y %H:%M:%S GMT')
            headers['Last-Modified']=otherStyleTime
            headers['ETag']=ETag
            return body,status,headers
        elif status:
            return Template(Temppath,tpldata=response.tpldata,**context),status,headers
        else:
            return Template(Temppath,tpldata=response.tpldata,**context),'200 ok',headers
    def json(res=[],status='200 ok',response_cache=False,ETag=None,header={"Content-Type":"application/json; charset=utf-8","Access-Control-Allow-Origin":"*"}):
        """响应json内容

        res  body内容

        status 响应状态码

        response_cache 是否启用浏览器缓存  响应状态码200 ok时有效

        ETag 缓存标识  响应状态码200 ok时有效

        header 响应头
        """
        headers=copy.deepcopy(header)
        if status=='200 ok' and response_cache:
            if not ETag:
                ETag=md5(globals.HEADER.URL)
            try:
                HTTP_IF_NONE_MATCH=globals.HEADER.GET['HTTP_IF_NONE_MATCH']
            except:
                HTTP_IF_NONE_MATCH=None
            if(HTTP_IF_NONE_MATCH and get_cache(ETag)):
                status="304 Not Modified"
                body=''
            else:
                if isinstance(response_cache,int) and response_cache>1:
                    set_cache(ETag,1,response_cache)
                    headers['response_cache']=str(response_cache)+" s"
                else:
                    set_cache(ETag,1)
                    headers['response_cache']="default"
                body=json_encode(res)
            dateArray = datetime.datetime.utcfromtimestamp(times()-86400)
            otherStyleTime = dateArray.strftime('%a, %d %b %Y %H:%M:%S GMT')
            headers['Last-Modified']=otherStyleTime
            headers['ETag']=ETag
            
        else:
            body=json_encode(res)
        return body,status,headers
    def pic(body,response_cache=True,ETag=None):
        """输出图片
        
        body 图片二进制内容或图片路径 建议使用图片路径

        response_cache 是否启用浏览器缓存  body使用图片路径时有效

        ETag 缓存标识
        """
        import filetype
        status='200 ok'
        header={}
        if response_cache:
            header={"Cache-Control":"public, max-age=2592000"}
            if not ETag:
                if body and isinstance(body,str):
                    ETag=md5(body+globals.HEADER.URL)
                else:
                    ETag=md5(globals.HEADER.URL)
            header['ETag']=ETag
        if body and isinstance(body,str):
            if response_cache:
                try:
                    HTTP_IF_NONE_MATCH=globals.HEADER.GET['HTTP_IF_NONE_MATCH']
                except:
                    HTTP_IF_NONE_MATCH=None
                if(HTTP_IF_NONE_MATCH and get_cache(ETag)):
                    status="304 Not Modified"
                    body=''
                else:
                    if isinstance(response_cache,int) and response_cache>1:
                        set_cache(ETag,1,response_cache)
                    else:
                        set_cache(ETag,1)
                    filename=body
                    f=open(filename,"rb")
                    body=f.read()
                    f.close()
                    kind = filetype.guess(filename)
                    try:
                        header['Content-Type']=kind.mime
                    except:
                        header['Content-Type']="image/png"
            else:
                filename=body
                f=open(filename,"rb")
                body=f.read()
                f.close()
                kind = filetype.guess(filename)
                try:
                    header['Content-Type']=kind.mime
                except:
                    header['Content-Type']="image/png"
            
        else:
            if response_cache:
                try:
                    HTTP_IF_NONE_MATCH=globals.HEADER.GET['HTTP_IF_NONE_MATCH']
                except:
                    HTTP_IF_NONE_MATCH=None
                if HTTP_IF_NONE_MATCH:
                    status="304 Not Modified"
                    body=''
            header['Content-Type']="image/png"
        dateArray = datetime.datetime.utcfromtimestamp(times()-86400)
        otherStyleTime = dateArray.strftime('%a, %d %b %Y %H:%M:%S GMT')
        header['Last-Modified']=otherStyleTime
        
        return body,status,header
    def video(body):
        """输出视频
        
        body 视频二进制内容或视频路径
        """
        import filetype
        status='200 ok'
        header={"Cache-Control":"public, max-age=2592000"}
        if isinstance(body,str):
            ETag=md5(body)
            try:
                HTTP_IF_NONE_MATCH=globals.HEADER.GET['HTTP_IF_NONE_MATCH']
            except:
                HTTP_IF_NONE_MATCH=None
            if(HTTP_IF_NONE_MATCH and get_cache(ETag)):
                header=get_cache(ETag)
                status="304 Not Modified"
                body=''
            else:
                filename=body
                f=open(filename,"rb")
                body=f.read()
                f.close()
                kind = filetype.guess(filename)
                try:
                    header['Content-Type']=kind.mime
                except:
                    header['Content-Type']="video/mp4"
                header['content-length']=str(len(body))
                set_cache(ETag,header,2592000)
            dateArray = datetime.datetime.utcfromtimestamp(times()-86400)
            otherStyleTime = dateArray.strftime('%a, %d %b %Y %H:%M:%S GMT')
            header['Last-Modified']=otherStyleTime
            header['ETag']=ETag
        else:
            header['Content-Type']="video/mp4"
        return body,status,header
    def audio(body):
        """输出音频
        
        body 音频二进制内容或音频路径
        """
        import filetype
        status='200 ok'
        header={"Cache-Control":"public, max-age=2592000"}
        if isinstance(body,str):
            ETag=md5(body)
            try:
                HTTP_IF_NONE_MATCH=globals.HEADER.GET['HTTP_IF_NONE_MATCH']
            except:
                HTTP_IF_NONE_MATCH=None
            if(HTTP_IF_NONE_MATCH and get_cache(ETag)):
                header=get_cache(ETag)
                status="304 Not Modified"
                body=''
            else:
                filename=body
                f=open(filename,"rb")
                body=f.read()
                f.close()
                kind = filetype.guess(filename)
                try:
                    header['Content-Type']=kind.mime
                except:
                    header['Content-Type']="audio/mpeg"
                header['content-length']=str(len(body))
                set_cache(ETag,header,2592000)
            dateArray = datetime.datetime.utcfromtimestamp(times()-86400)
            otherStyleTime = dateArray.strftime('%a, %d %b %Y %H:%M:%S GMT')
            header['Last-Modified']=otherStyleTime
            header['ETag']=ETag
        else:
            header['Content-Type']="audio/mpeg"
        return body,status,header
    def download(pathname):
        """下载文件
        
        pathname 文件路径
        """
        import filetype
        if os.path.isfile(pathname):
            f=open(pathname,"rb")
            body=f.read()
            f.close()
            kind = filetype.guess(pathname)
            try:
                return body,"200 ok",{"Content-Type":"application/"+kind.mime,"Accept-Ranges":"bytes"}
            except:
                return body,"200 ok",{"Content-Type":"application/text","Accept-Ranges":"bytes"}
        else:
            return Templates('E:\doc\python\kwebsp\kwebs/tpl/err.html',title="文件不存在",content="文件不存在",imgsrc=config.domain['kcwebsimg']+"/icon/error.png",config=config)
    def redirect(url,status="302 Found",html='',header={"Content-Type":"application/html; charset=utf-8"}):
        """重定向

        参数 url 重定向地址 必须

        参数 status 响应码  可选

        参数 html body响应内容 可选

        参数 header 响应头  可选
        """
        header['Location']=url
        return html,status,header
    
if 'Linux' in get_sysinfo()['platform']:
    #添加自启命令
    if not os.path.isfile('/usr/bin/startkwebs.sh'):
        open('/usr/bin/startkwebs.sh', 'w').close()
        t=file_get_content("/etc/os-release")
        if 'centos' in t.lower():
            os.system("sed -i 's/bash startkwebs.sh//g' /etc/rc.d/rc.local")
            os.system("echo 'bash startkwebs.sh'  >> /etc/rc.d/rc.local")
            os.system('chmod 777 /etc/rc.d/rc.local')
            os.system('chmod 777 /usr/bin/startkwebs.sh')
        elif 'ubuntu' in t.lower():
            if not os.path.isfile("/etc/rc.local"):
                file_set_content('/etc/rc.local','#!/bin/sh')
            os.system("sed -i 's/bash startkwebs.sh//g' /etc/rc.local")
            os.system("echo 'bash startkwebs.sh'  >> /etc/rc.local")
            os.system('sudo chmod +x /etc/rc.local')
            os.system('sudo chmod 777 /usr/bin/startkwebs.sh')
def insert_system_up(cmd):
    """添加开机启动命令
    
    cmd 命令
    """
    if 'Linux' in get_sysinfo()['platform']:
        f=open("/usr/bin/startkwebs.sh","a")
        f.write("\n"+cmd+"\n")
        f.close()
        return True
    else:
        raise Exception('暂不支持linux以外的系统')
def del_system_up(cmd,vague=False):
    """删除开机启动命令
    
    cmd 命令

    vague 是否模糊匹配 
    """
    if 'Linux' in get_sysinfo()['platform']:
        if vague:
            f = open("/usr/bin/startkwebs.sh")
            con=''
            while True:
                line = f.readline()
                if not line:
                    break
                if cmd in line:
                    line=''
                con=con+line
            f.close()
            file_set_content("/usr/bin/startkwebs.sh",con)
        else:
            content=file_get_content("/usr/bin/startkwebs.sh")
            content=content.replace("\n"+cmd+"\n","")
            file_set_content("/usr/bin/startkwebs.sh",content)
        return True
    else:
        raise Exception('暂不支持linux以外的系统')
def rfdsgeagesegdsfdsfdrrsebskwebsafterrequest():
    #关闭数据库连接 不要修改改方法，也不要定义与该方法相同名字的方法
    mysqldbobj.close()


# -*- coding: utf-8 -*-
from kwebs.config import session as kcwssession
from . import globals as kcwsglobals
import time,random,hashlib,kccache
from datetime import datetime
def __md5(strs):
    m = hashlib.md5()
    m.update(strs.encode())
    return m.hexdigest()
def set(name,value,expire=None):
    "设置session"
    if not expire:
        expire=kcwssession['expire']
    HTTP_COOKIE=kcwsglobals.HEADER.HTTP_COOKIE
    SESSIONID="SESSIONID"+__md5(str(name)+str(kcwssession['prefix']))[0:8]  #######
    try: 
        HTTP_COOKIE=HTTP_COOKIE.split(";")
    except:
        token=None
    else:
        token=None
        for k in HTTP_COOKIE:
            if SESSIONID in k:
                token=k.split("=")[1]
    if not token:
        strs="kcws"+str(time.time())+str(random.randint(0,9))
        token=__md5(strs)
    kcwsglobals.set_cookie=SESSIONID+"="+token+";expires="+datetime.utcnow().strftime('%a, %d %b %Y %H:%M:%S GMT')+"; Max-Age=%d;Path=/" % (int(expire)*10)
    return kccache.cache.set_config(kcwssession).set_cache(token,value,expire)
def get(name):
    "获取session"
    HTTP_COOKIE=kcwsglobals.HEADER.HTTP_COOKIE
    try:
        HTTP_COOKIE=HTTP_COOKIE.split(";")
    except:
        return None
    SESSIONID="SESSIONID"+__md5(str(name)+str(kcwssession['prefix']))[0:8]  #########
    token=''
    for k in HTTP_COOKIE:
        if SESSIONID in k:
            token=k.split("=")[1]
    v=kccache.cache.set_config(kcwssession).get_cache(token)
    return v
def rm(name):
    "删除session"
    HTTP_COOKIE=kcwsglobals.HEADER.HTTP_COOKIE
    try:
        HTTP_COOKIE=HTTP_COOKIE.split(";")
    except:
        return None
    SESSIONID="SESSIONID"+__md5(str(name)+str(kcwssession['prefix']))[0:8]  #######
    token=''
    for k in HTTP_COOKIE:
        if SESSIONID in k:
            token=k.split("=")[1]
    kccache.cache.set_config(kcwssession).del_cache(token)
    kcwsglobals.set_cookie=SESSIONID+"="+token+";expires="+datetime.utcnow().strftime('%a, %d %b %Y %H:%M:%S GMT')+"; Max-Age=2"
    return True




# -*- coding: utf-8 -*-
from .database import *
app['app_debug']=True  #是否开启调试模式
app['tpl_folder']='./app'  #设置模板文件目录名 注意：所有的配置目录都是以您的运行文件所在目录开始
app['before_request']='before_request'  #设置请求前要执行的函数名
app['after_request']='after_request'    #设置请求后要执行的函数名
app['staticpath']='app/static'          #静态主要目录
app['appmode']='develop'  #produc 生产环境  develop 开发环境

# session配置
session['type']='File' #session 存储类型  支持 File,Redis
session['path']='./app/runtime/session/temp' #session缓存目录
session['expire']=86400 #session默认有效期 该时间是指session在服务的保留时间，通常情况下浏览器上会保留该值的10倍
session['prefix']="kcws" # SESSION 前缀
session['host']=redis['host'] #Redis服务器地址
session['port']=redis['port'] #Redis 端口
session['password']=redis['password'] #Redis登录密码
session['db']=1 #Redis数据库    注：Redis用1或2或3等表示

#缓存配置
cache['type']='File' #驱动方式 支持 File,Redis
cache['path']='./app/runtime/cachepath' #缓存保存目录 
cache['expire']=120 #缓存有效期 0表示永久缓存
cache['host']=redis['host'] #Redis服务器地址
cache['port']=redis['port'] #Redis 端口
cache['password']=redis['password'] #Redis登录密码
cache['db']=2 #Redis数据库    注：Redis用1或2或3等表示

#email配置
email['sender']='' #发件人邮箱账号
email['pwd']='' #发件人邮箱密码(如申请的smtp给的口令)
email['sendNick']='' #发件人昵称
email['theme']='' #默认主题
email['recNick']='' #默认收件人昵称

#路由配置
route['default']=True #是否开启默认路由  默认路由开启后面不影响以下配置的路由，模块名/版本名/控制器文件名/方法名 作为路由地址   如：http://www.kcws.com/modular/plug/index/index/
route['defmodular']='api' #默认模块 当url不包括模块名时
route['defplug']='index' #默认插件 当url不包括插件名时
route['files']='index' #默认路由文件（控制器） 当url不包括控制器名时
route['funct']='index'  #默认路由函数 (操作方法) 当url不包括操作方法名时
route['methods']=['POST','GET','DELETE','PUT'] #默认请求方式






# -*- coding: utf-8 -*-
from .other import *
redis['host']='127.0.0.1' #服务器地址
redis['port']=6379 #端口
redis['password']=''  #密码
redis['db']=0 #Redis数据库    注：Redis用0或1或2等表示
redis['pattern']=True # True连接池链接 False非连接池链接
redis['ex']=0  #过期时间 （秒）




# -*- coding: utf-8 -*-
from kwebs import config
import kcsqlite
class model:
    table=None
    fields={}
    __db=None
    config=config.database
    def __init__(self):
        if not self.table:
            self.table=self.__class__.__name__
        self.__db=kcsqlite.sqlite.connect(self.config).table(self.table)
    def create_table(self):
        "创建表"
        sqlist=[]
        for k in self.fields.keys():
            sqlist.append(k+" "+self.fields[k])
        # print(self.table)
        sqls="create table "+self.table+" ("
        for k in sqlist:
            sqls=sqls+k+", "
        sqls=sqls[:-2]+")"
        # print(sqls)
        self.__db.execute(sqls)
    def find(self):
        return self.__db.find()
    def select(self):
        lists=self.__db.select()
        # print(lists)
        return lists
    def insert(self,data):
        return self.__db.insert(data)
    def update(self,data):
        return self.__db.update(data)
    def startTrans(self):
        "开启事务,仅对 update方法、delete方法、install方法有效"
        self.__db.startTrans()
    def commit(self):
        """事务提交

        增删改后的任务进行提交
        """
        self.__db.commit()
    def rollback(self):
        """事务回滚

        增删改后的任务进行撤销
        """
        self.__db.rollback()
    def where(self,where = None,*wheres):
        """设置过滤条件

        传入方式:
        "id",2 表示id='2'

        "id","in",2,3,4,5,6,...表示 id in (2,3,4,5,6,...)

        "id","or",2,3,4,5,6,...表示 id=2 or id=3 or id=4...

        [("id","gt",6000),"and",("name","like","%超")] 表示 ( id > "6000" and name LIKE "%超" )

        "id","eq",1 表示 id = '1'

        eq 等于
            neq 不等于
            gt 大于
            egt 大于等于
            lt 小于
            elt 小于等于
            like LIKE
        """
        self.__db.where(where,*wheres)
        return self
    def field(self,field = "*"):
        """设置过滤显示条件

        参数 field：str 字符串
        """
        self.__db.field(field)
        return self
    __limit=[]
    def limit(self,offset, length = None):
        """设置查询数量

        参数 offset：int 起始位置

        参数 length：int 查询数量
        """
        self.__db.limit(offset, length)
        return self
    def order(self,strs=None,*strs1):
        """设置排序查询

        传入方式:

        "id desc"

        "id",'name','appkey','asc'

        "id",'name','appkey'   不包含asc或desc的情况下 默认是desc

        ['id','taskid',{"task_id":"desc"}]
        """
        self.__db.order(strs=None,*strs1)
        return self
    __distinct=None
    def distinct(self,bools=None):
        "用于返回唯一不同的值,配合field方法使用生效,消除所有重复的记录，并只获取唯一一次记录。"
        self.__db.distinct(bools)
        return self
    def deltableall(self):
        "删除当前数据库所有表格 mysql有效"
        if self.conf['type']=='mysql':
            a=self.__db.execute("SELECT concat('DROP TABLE IF EXISTS ', table_name, ';') FROM information_schema.tables WHERE table_schema = 'core1';")
            for k in a:
                self.__db.execute(k["concat('DROP TABLE IF EXISTS ', table_name, ';')"])
        



class dbtype:
    conf=model.config
    def int(LEN=16,DEFAULT=False,NULL=False,UNIQUE=False,PRI=False,A_L=False):
        # print(dbtype.conf['type'])
        if dbtype.conf['type']=='mysql':
            strs="INT("+str(LEN)+")"
            if DEFAULT:
                strs=strs+" DEFAULT "+str(DEFAULT)
            if NULL:
                strs=strs+" NULL"
            else:
                strs=strs+" NOT NULL"
            if UNIQUE:
                strs=strs+" UNIQUE"
            if PRI:
                strs=strs+" PRIMARY KEY"
            if A_L:
                strs=strs+" AUTO_INCREMENT"
        else:
            strs="INTEGER"
            if DEFAULT:
                strs=strs+" DEFAULT "+str(DEFAULT)
            if NULL:
                strs=strs+" NULL"
            else:
                strs=strs+" NOT NULL"
            if UNIQUE:
                strs=strs+" UNIQUE"
            if PRI:
                strs=strs+" PRIMARY KEY"
            if A_L:
                strs=strs+" AUTOINCREMENT"
        return strs
    def varchar(LEN=32,DEFAULT=False,NULL=False,UNIQUE=False,INDEX=False,FULLTEXT=False):
        strs="VARCHAR("+str(LEN)+")"
        if DEFAULT:
            strs=strs+" DEFAULT "+str(DEFAULT)
        elif DEFAULT=='':
            strs=strs+" DEFAULT ''"
        if NULL:
            strs=strs+" NULL"
        else:
            strs=strs+" NOT NULL"
        if UNIQUE:
            strs=strs+" UNIQUE"
        if INDEX:
            strs=strs+" INDEX"
        if FULLTEXT:
            strs=strs+" FULLTEXT"
        return strs
    def text(NULL=False):
        if dbtype.conf['type']=='mysql':
            strs="TEXT CHARACTER SET utf8 COLLATE utf8_general_ci"
        else:
            strs="TEXT"
        if NULL:
            strs=strs+" NULL"
        else:
            strs=strs+" NOT NULL"
        return strs
    def char(LEN=16,DEFAULT=False,NULL=False,UNIQUE=False,INDEX=False):
        strs=" CHAR("+str(LEN)+")"
        if DEFAULT:
            strs=strs+" DEFAULT "+str(DEFAULT)
        elif DEFAULT=='':
            strs=strs+" DEFAULT ''"
        if NULL:
            strs=strs+" NULL"
        else:
            strs=strs+" NOT NULL"
        if UNIQUE:
            strs=strs+" UNIQUE"
        if INDEX:
            strs=strs+" INDEX"
        return strs
    def decimat(LEN="10,2",DEFAULT=False,NULL=False,UNIQUE=False,INDEX=False):
        "小数类型"
        strs="DECIMAL("+str(LEN)+")"
        if DEFAULT:
            strs=strs+" DEFAULT "+str(DEFAULT)
        elif DEFAULT=='':
            strs=strs+" DEFAULT ''"
        if NULL:
            strs=strs+" NULL"
        else:
            strs=strs+" NOT NULL"
        if UNIQUE:
            strs=strs+" UNIQUE"
        if INDEX:
            strs=strs+" INDEX"
        return strs
    def date(NULL=False):
        strs=" DATE"
        if NULL:
            strs=strs+" NULL"
        else:
            strs=strs+" NOT NULL"
        return strs


# -*- coding: utf-8 -*-
from .redis import *
database['type']='mysql' # 数据库类型  目前支持mysql和sqlite
database['debug']=False  #是否开启数据库调试描述
database['host']=['127.0.0.1']#服务器地址 [地址1,地址2,地址3...] 多个地址分布式(主从服务器)下有效
database['port']=[3306] #端口 [端口1,端口2,端口3...]
database['user']=['root']  #用户名 [用户名1,用户名2,用户名3...]
database['password']=['root']  #密码 [密码1,密码2,密码3...]
database['db']=['test']  #数据库名 [数据库名1,数据库名2,数据库名3...]
database['charset']='utf8mb4'   #数据库编码默认采用utf8mb4
database['pattern']=False # True数据库长连接模式 False数据库短连接模式  注：建议web应用有效，cli应用方式下，如果长时间运行建议使用mysql().close()关闭
database['cli']=False # 是否以cli方式运行
database['dbObjcount']=1 # 连接池数量（单个数据库地址链接数量），数据库链接实例数量 mysql长链接模式下有效
database['deploy']=0 # 数据库部署方式:0 集中式(单一服务器),1 分布式(主从服务器)  mysql数据库有效
database['master_num']=1 #主服务器数量 不能超过host服务器数量  （等于服务器数量表示读写不分离：主主复制。  小于服务器表示读写分离：主从复制。） mysql数据库有效
database['master_dql']=False #主服务器是否可以执行dql语句 是否可以执行select语句  主服务器数量大于等于host服务器数量时必须设置True
database['break']=0 #断线重连次数，0表示不重连。 注：cli模式下 10秒进行一次重连并且连接次数是当前配置的300倍

#mongodb配置
mongo['host']='127.0.0.1'
mongo['port']='27017'
mongo['user']=''
mongo['password']=''
mongo['db']='test'
mongo['retryWrites']=False #是否支持重新写入

#sqlite配置
sqlite['db']='kcwslicuxweb' #sqlite数据库文件


from kunapi.config import *
from .. import kwebsinfo as kwebs
# redis配置
redis={}
redis['host']='127.0.0.1' #服务器地址
redis['port']=6379 #端口
redis['password']=''  #密码
redis['db']=0 #Redis数据库    注：Redis用0或1或2等表示
redis['pattern']=True # True连接池链接 False非连接池链接
redis['ex']=0  #过期时间 （秒）
redis['debug']=False
redis['th_lock']=False

#缓存配置
cache={}
cache['type']='File' #驱动方式 支持 File Redis Python
cache['path']='app/runtime/cachepath' #缓存保存目录 
cache['expire']=120 #缓存有效期 0表示永久缓存
cache['host']=redis['host'] #Redis服务器地址
cache['port']=redis['port'] #Redis 端口
cache['password']=redis['password'] #Redis登录密码
cache['db']=1 #Redis数据库    注：Redis用1或2或3等表示
cache['debug']=False
cache['th_lock']=False
cachevalue={}

# session配置
session={}
session['type']='File' #session 存储类型  支持 file、Redis、Python
session['path']='app/runtime/session' #session缓存目录
session['expire']=86400 #session默认有效期 该时间是指session在服务的保留时间，通常情况下浏览器上会保留该值的10倍
session['prefix']="kcws" # SESSION 前缀
session['host']=redis['host'] #Redis服务器地址
session['port']=redis['port'] #Redis 端口
session['password']=redis['password'] #Redis登录密码
session['db']=2 #Redis数据库    注：Redis用1或2或3等表示
session['th_lock']=False
session['debug']=False


# 默认数据库配置
database={}
database['type']='mysql' # 数据库类型  目前支持mysql和sqlite
database['debug']=False  #是否开启数据库调试描述
database['host']=['127.0.0.1']#服务器地址 [地址1,地址2,地址3...] 多个地址分布式(主从服务器)下有效
database['port']=[3306] #端口 [端口1,端口2,端口3...]
database['user']=['root']  #用户名 [用户名1,用户名2,用户名3...]
database['password']=['root']  #密码 [密码1,密码2,密码3...]
database['db']=['test']  #数据库名 [数据库名1,数据库名2,数据库名3...]
database['charset']='utf8mb4'   #数据库编码默认采用utf8mb4
database['pattern']=False # True数据库长连接模式 False数据库短连接模式  注：建议web应用有效，cli应用方式下，如果长时间运行建议使用mysql().close()关闭
database['cli']=False # 是否以cli方式运行
database['dbObjcount']=1 # 连接池数量（单个数据库地址链接数量），数据库链接实例数量 mysql长链接模式下有效
database['deploy']=0 # 数据库部署方式:0 集中式(单一服务器),1 分布式(主从服务器)  mysql数据库有效
database['master_num']=1 #主服务器数量 不能超过host服务器数量  （等于服务器数量表示读写不分离：主主复制。  小于服务器表示读写分离：主从复制。） mysql数据库有效
database['master_dql']=False #主服务器是否可以执行dql语句 是否可以执行select语句  主服务器数量大于等于host服务器数量时必须设置True
database['break']=0 #断线重连次数，0表示不重连。 注：cli模式下 10秒进行一次重连并且连接次数是当前配置的300倍
database['autocommit']=False #自动提交查询事务  命令行运行时开启 web运行时关闭
database['th_lock']=False

#sqlite配置
sqlite={}
sqlite['db']='kcwsdb'  # 数据库文件存放地址
sqlite['debug']=False
sqlite['th_lock']=False

#mongodb配置
mongo={}
mongo['host']='127.0.0.1'
mongo['port']='27017'
mongo['user']=''
mongo['password']=''
mongo['db']='test'
mongo['retryWrites']=False #是否支持重新写入


#email配置
email={}
email['sender']='' #发件人邮箱账号
email['pwd']='' #发件人邮箱密码(如申请的smtp给的口令)
email['sendNick']='' #发件人昵称
email['theme']='' #默认主题
email['recNick']='' #默认收件人昵称


fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr=kwebs['name'] #不要修改该参数，否则无法上传模块和插件




from .common import *
from kunapi import kunapi
import os,shutil
def cill_start(fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr='kwebs'):
    "脚本入口"
    
    cmd_par=kunapi.get_cmd_par(fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr=fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr)
    if cmd_par and not cmd_par['project']:
        cmd_par['project']='kwebs'
    if cmd_par and cmd_par['install'] and not cmd_par['help']:#插入 应用、模块、插件
        if cmd_par['appname']:
            remppath=os.path.split(os.path.realpath(__file__))[0]
            if not os.path.exists(cmd_par['project']+'/'+cmd_par['appname']) and not os.path.exists(cmd_par['appname']):
                shutil.copytree(remppath+'/tempfile/kwebs',cmd_par['project'])
                print('kwebs项目创建成功')
            else:
                return kunapi.cill_start(fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr=fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr)
        else:
            return kunapi.cill_start(fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr=fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr)
    elif cmd_par:
        return kunapi.cill_start(fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr=fdgrsgrsegsrsgrsbsdbftbrsbfdrtrtbdfsrsgr)


#项目运行文件，请务修改
import kunapi,sys,app
app=kunapi.web(__name__,app)
if __name__ == "__main__":
    try:
        route=sys.argv[1]
        if "eventlog"==route:
            raise Exception("")
    except:
        #host监听ip port端口 name python解释器名字 (windows一般是python  linux一般是python3) 
        app.run(host="0.0.0.0",port="39001",name="python3.8")
    else:
        app.cli(route)



# -*- coding: utf-8 -*-
__version__ = '1.1'

kwebsinfo={}
kwebsinfo['name']='kwebs'                             #项目的名称
kwebsinfo['version']=__version__							#项目版本
kwebsinfo['description']='kwebs作为web开发而设计的高性能框架'       #项目的简单描述
kwebsinfo['long_description']='kwebs作为web开发而设计的高性能框架，采用全新的架构思想，注重易用性。遵循MIT开源许可协议发布，意味着个人和企业可以免费使用kwebs，甚至允许把你基于kwebs开发的应用开源或商业产品发布或销售。完整文档请访问：https://docs.kwebapp.cn/index/index/2'     #项目详细描述
kwebsinfo['license']='MIT License'                    #开源协议   mit开源
kwebsinfo['url']='https://docs.kwebapp.cn/index/index/2'
kwebsinfo['author']='百里-坤坤'  					 #名字
kwebsinfo['author_email']='kcweb@kwebapp.cn' 	     #邮件地址
kwebsinfo['maintainer']='坤坤' 						 #维护人员的名字
kwebsinfo['maintainer_email']='fk1402936534@qq.com'    #维护人员的邮件地址
kwebsinfo['username']=''
kwebsinfo['password']=''


from queue import Queue
from .db import model
import threading,time,os,hashlib,random,traceback,multiprocessing,kcsqlite
queuesdbpath=os.path.split(os.path.realpath(__file__))[0]+"/Queues"
class model_kwebs_task(model.model):
    "任务"
    config={'type':'sqlite','db':queuesdbpath}
    model.dbtype.conf=config
    table="model_kwebs_queues" 
    fields={
        "id":model.dbtype.int(LEN=11,PRI=True,A_L=True),        #设置id为自增主键
        "taskid":model.dbtype.varchar(LEN=32,DEFAULT=''),        #设置id为自增主键
        "title":model.dbtype.varchar(LEN=1024,DEFAULT=''),      #名称
        "describes":model.dbtype.varchar(LEN=2048,DEFAULT=''),  #描述
        "code":model.dbtype.int(LEN=11,DEFAULT=2),              #状态码 0成功 1失败 2等待中 3正在执行  4完成
        "msg":model.dbtype.text(),                              #状态描述
        "error":model.dbtype.text(),                            #异常信息
        "start":model.dbtype.varchar(LEN=11,DEFAULT=0),             #进度条起始值
        "end":model.dbtype.int(LEN=11,DEFAULT=100),               #进度条结束值
        "starts":model.dbtype.varchar(LEN=11,DEFAULT=0),             #每秒钟进度条起始值增加多少
        "addtime":model.dbtype.int(LEN=11,DEFAULT=0),            #添加时间
        "endtime":model.dbtype.int(LEN=11,DEFAULT=0),            #结束时间
        "updtime":model.dbtype.int(LEN=11,DEFAULT=0)            #更新时间
    }
class Queues():
    __globalqueue=None
    __processglobalqueue=None
    def __start():
        if not Queues.__globalqueue:
            if not os.path.isfile(queuesdbpath):
                model_kwebs_tasks=model_kwebs_task()
                model_kwebs_tasks.create_table()
            try:
                kcsqlite.sqlite.connect(queuesdbpath,th_lock=True).table("model_kwebs_queues").find()
            except:
                model_kwebs_tasks=model_kwebs_task()
                model_kwebs_tasks.create_table()
            Queues.__globalqueue=Queue()
            t=threading.Thread(target=Queues.__messagequeue,daemon=True)
            t.start()
    __scripttask=None
    __scripttasklist=[]
    def __script():
        if not Queues.__scripttask:
            if not os.path.isfile(queuesdbpath):
                model_kwebs_tasks=model_kwebs_task()
                model_kwebs_tasks.create_table()
            try:
                kcsqlite.sqlite.connect(queuesdbpath,th_lock=True).table("model_kwebs_queues").find()
            except:
                model_kwebs_tasks=model_kwebs_task()
                model_kwebs_tasks.create_table()
            Queues.__scripttask=Queue()
            t=threading.Thread(target=Queues.__taskmessagequeue,daemon=True)
            t.start()
            t=threading.Thread(target=Queues.__taskmessagequeue_zx,daemon=True)
            t.start()
    
    def __taskmessagequeue_zx():
        time.sleep(10)
        while True:
            for value in Queues.__scripttasklist:
                try:
                    kcsqlite.sqlite.connect(queuesdbpath,th_lock=True).table("model_kwebs_queues").where("taskid = '"+value['task']['taskid']+"' and code!=4").update({"code":3,"error":""})
                except:
                    pass
                else:
                    if value['args']:
                        try:
                            value['target'](*value['args'])
                        except:
                            kcsqlite.sqlite.connect(queuesdbpath,th_lock=True).table("model_kwebs_queues").where("taskid = '"+value['task']['taskid']+"' and code!=4").update({"code":1,'endtime':int(time.time()),'updtime':int(time.time()),"error":str(traceback.format_exc())})
                        else:
                            kcsqlite.sqlite.connect(queuesdbpath,th_lock=True).table("model_kwebs_queues").where("taskid = '"+value['task']['taskid']+"' and code!=4").update({"code":4,"start":100,'endtime':int(time.time()),'updtime':int(time.time())})
                    else:
                        try:
                            value['target']()
                        except:
                            kcsqlite.sqlite.connect(queuesdbpath,th_lock=True).table("model_kwebs_queues").where("taskid = '"+value['task']['taskid']+"' and code!=4").update({"code":1,'endtime':int(time.time()),'updtime':int(time.time()),"error":str(traceback.format_exc())})
                        else:
                            kcsqlite.sqlite.connect(queuesdbpath,th_lock=True).table("model_kwebs_queues").where("taskid = '"+value['task']['taskid']+"' and code!=4").update({"code":4,"start":100,'endtime':int(time.time()),'updtime':int(time.time())})
            time.sleep(1)
    def __taskmessagequeue():
        kcsqlite.sqlite.connect(queuesdbpath,th_lock=True).table("model_kwebs_queues").where("code=2 and updtime<"+str(int(time.time())-86400*7)).delete()
        kcsqlite.sqlite.connect(queuesdbpath,th_lock=True).table("model_kwebs_queues").where("code=3 and updtime<"+str(int(time.time())-1200)).delete()
        while True:
            if not Queues.__scripttask.empty():
                value=Queues.__scripttask.get()
                Queues.__scripttasklist.append(value)
            else:
                time.sleep(0.01)
    def __startprocess():
        if not Queues.__processglobalqueue:
            if not os.path.isfile(queuesdbpath):
                model_kwebs_tasks=model_kwebs_task()
                model_kwebs_tasks.create_table()
            try:
                kcsqlite.sqlite.connect(queuesdbpath,th_lock=True).table("model_kwebs_queues").find()
            except:
                model_kwebs_tasks=model_kwebs_task()
                model_kwebs_tasks.create_table()
            Queues.__processglobalqueue=multiprocessing.Queue()
            t=multiprocessing.Process(target=Queues._messagequeueprocess,args=(Queues.__processglobalqueue,),daemon=True)
            # t.daemon=True
            t.start()
    def _messagequeueprocess(processglobalqueue):
        """不支持外部调用"""
        kcsqlite.sqlite.connect(queuesdbpath,th_lock=True).table("model_kwebs_queues").where("code=2 and updtime<"+str(int(time.time())-86400*7)).delete()
        kcsqlite.sqlite.connect(queuesdbpath,th_lock=True).table("model_kwebs_queues").where("code=3 and updtime<"+str(int(time.time())-1200)).delete()
        pid = os.getpid()
        pdfile=os.path.split(os.path.realpath(__file__))[0][:-6]+'/pid/queues_pid_'+str(pid) #pid存放文件
        f=open(pdfile,'w')
        f.write(str(pid))
        f.close()
        while True:
            if not processglobalqueue.empty():
                value=processglobalqueue.get()
                kcsqlite.sqlite.connect(queuesdbpath,th_lock=True).table("model_kwebs_queues").where("taskid = '"+value['task']['taskid']+"' and code!=4").update({"code":3,"error":""})
                if value['args']:
                    try:
                        value['target'](*value['args'])
                    except:
                        kcsqlite.sqlite.connect(queuesdbpath,th_lock=True).table("model_kwebs_queues").where("taskid = '"+value['task']['taskid']+"' and code!=4").update({"code":1,'endtime':int(time.time()),'updtime':int(time.time()),"error":str(traceback.format_exc())})
                    else:
                        kcsqlite.sqlite.connect(queuesdbpath,th_lock=True).table("model_kwebs_queues").where("taskid = '"+value['task']['taskid']+"' and code!=4").update({"code":4,"start":100,'endtime':int(time.time()),'updtime':int(time.time())})
                else:
                    try:
                        value['target']()
                    except:
                        kcsqlite.sqlite.connect(queuesdbpath,th_lock=True).table("model_kwebs_queues").where("taskid = '"+value['task']['taskid']+"' and code!=4").update({"code":1,'endtime':int(time.time()),'updtime':int(time.time()),"error":str(traceback.format_exc())})
                    else:
                        kcsqlite.sqlite.connect(queuesdbpath,th_lock=True).table("model_kwebs_queues").where("taskid = '"+value['task']['taskid']+"' and code!=4").update({"code":4,"start":100,'endtime':int(time.time()),'updtime':int(time.time())})
            else:
                time.sleep(0.01)
    def __messagequeue():
        kcsqlite.sqlite.connect(queuesdbpath,th_lock=True).table("model_kwebs_queues").where("code=2 and updtime<"+str(int(time.time())-86400*7)).delete()
        kcsqlite.sqlite.connect(queuesdbpath,th_lock=True).table("model_kwebs_queues").where("code=3 and updtime<"+str(int(time.time())-1200)).delete()
        while True:
            if not Queues.__globalqueue.empty():
                value=Queues.__globalqueue.get()
                kcsqlite.sqlite.connect(queuesdbpath,th_lock=True).table("model_kwebs_queues").where("taskid = '"+value['task']['taskid']+"' and code!=4").update({"code":3,"error":""})
                if value['args']:
                    try:
                        value['target'](*value['args'])
                    except:
                        kcsqlite.sqlite.connect(queuesdbpath,th_lock=True).table("model_kwebs_queues").where("taskid = '"+value['task']['taskid']+"' and code!=4").update({"code":1,'endtime':int(time.time()),'updtime':int(time.time()),"error":str(traceback.format_exc())})
                    else:
                        kcsqlite.sqlite.connect(queuesdbpath,th_lock=True).table("model_kwebs_queues").where("taskid = '"+value['task']['taskid']+"' and code!=4").update({"code":4,"start":100,'endtime':int(time.time()),'updtime':int(time.time())})
                else:
                    try:
                        value['target']()
                    except:
                        kcsqlite.sqlite.connect(queuesdbpath,th_lock=True).table("model_kwebs_queues").where("taskid = '"+value['task']['taskid']+"' and code!=4").update({"code":1,'endtime':int(time.time()),'updtime':int(time.time()),"error":str(traceback.format_exc())})
                    else:
                        kcsqlite.sqlite.connect(queuesdbpath,th_lock=True).table("model_kwebs_queues").where("taskid = '"+value['task']['taskid']+"' and code!=4").update({"code":4,"start":100,'endtime':int(time.time()),'updtime':int(time.time())})
            else:
                time.sleep(0.01)
    def delhist():
        """清除任务历史记录(包括 成功的 失败的 已完成的)"""
        return kcsqlite.sqlite.connect(queuesdbpath,th_lock=True).table("model_kwebs_queues").where("code","in",0,1,4).delete()
    def delwhere(where):
        "通过where条件删除 (不推荐使用)"
        return kcsqlite.sqlite.connect(queuesdbpath,th_lock=True).table("model_kwebs_queues").where(where).delete()
    def seltitle(title):
        "通过标题查询 (不推荐使用)"
        try:
            return kcsqlite.sqlite.connect(queuesdbpath,th_lock=True).table("model_kwebs_queues").where("title='"+title+"'").select()
        except:
            return ''
    def setfield(taskid,key,value):
        """设置指定字段(不建议使用)"""
        try:
            return kcsqlite.sqlite.connect(queuesdbpath,th_lock=True).table("model_kwebs_queues").where("taskid",taskid).update({key:value})
        except:
            return False
    def setstart(taskid,start=0.001,describes=None):
        """增加进度条起始位置
        
        start 支持0.001到10
        """
        if start>=0.001 and start<=10:
            arr=kcsqlite.sqlite.connect(queuesdbpath,th_lock=True).table("model_kwebs_queues").where("taskid",taskid).find()
            if arr:
                start+=round(float(arr['start']),3)
                if start>=0.01 and start <=99.99:
                    dqsjc=int(time.time())-arr['updtime'] #当前时间差
                    if dqsjc>=1:
                        starts=round(float((float(start)-float(arr['start']))/dqsjc),3)
                        upddate={"start":start,'starts':starts,'updtime':int(time.time())}
                        if describes:
                            upddate['describes']=describes
                        kcsqlite.sqlite.connect(queuesdbpath,th_lock=True).table("model_kwebs_queues").where("taskid",taskid).update(upddate)
                    return True
                else:
                    return False
            else:
                return False
        else:
            False
    def insert(target,args=None,title="默认任务",describes='',msg='',taskid=None,start=0,updtime=0,types=''): #add_queue
        """添加队列
        
        target 方法名  必须

        args 方法参数 非必须  如 (参数1,参数2)

        title 任务名称

        describes 任务描述

        msg 状态描述

        taskid 任务id

        start 进度条开始位置 （建议1到50）

        updtime 进度条更新时间

        types process表示使用进程执行 否则使用线程执行

        return taskid
        """
        if types=='process':
            Queues.__startprocess()
        else:
            Queues.__start()
        ttt=int(time.time())
        
        end=100
        if not updtime:
            updtime=ttt
        if not taskid:
            m = hashlib.md5()
            m.update((str(ttt)+str(random.randint(100000,999999))).encode(encoding='utf-8'))
            taskid=m.hexdigest()
        else:
            arr=kcsqlite.sqlite.connect(queuesdbpath,th_lock=True).table("model_kwebs_queues").where("taskid",taskid).find()
            if arr:
                if arr['code'] in [2,3]:
                    return taskid
                else:
                    kcsqlite.sqlite.connect(queuesdbpath,th_lock=True).table("model_kwebs_queues").where("taskid",taskid).delete()
        task={"taskid":taskid,"title":title,"describes":describes,"code":2,"msg":msg,"error":"","start":start,"end":end,"addtime":ttt,'starts':0,'endtime':0,'updtime':updtime}
        kcsqlite.sqlite.connect(queuesdbpath,th_lock=True).table("model_kwebs_queues").insert(task)
        key={"target":target,"args":args,"task":task}
        if types=='process':
            Queues.__processglobalqueue.put(key)
        else:
            Queues.__globalqueue.put(key)
        return taskid
    def inserttask(target,args=None,title="默认任务",msg='',taskid=None,start=0,updtime=0): #add_queue
        """添加后台任务
        
        target 方法名  必须

        args 方法参数 非必须  如 (参数1,参数2)

        title 任务名称

        describes 任务描述

        msg 状态描述

        taskid 任务id

        start 进度条开始位置 （建议1到50）

        updtime 进度条更新时间

        return taskid
        """
        Queues.__script()
        ttt=int(time.time())
        
        end=100
        if not updtime:
            updtime=ttt
        if not taskid:
            m = hashlib.md5()
            m.update((str(ttt)+str(random.randint(100000,999999))).encode(encoding='utf-8'))
            taskid=m.hexdigest()
        else:
            arr=kcsqlite.sqlite.connect(queuesdbpath,th_lock=True).table("model_kwebs_queues").where("taskid",taskid).find()
            if arr:
                return taskid
        describes='后台任务脚本'
        task={"taskid":taskid,"title":title,"describes":describes,"code":2,"msg":msg,"error":"","start":start,"end":end,"addtime":ttt,'starts':0,'endtime':0,'updtime':updtime}
        kcsqlite.sqlite.connect(queuesdbpath,th_lock=True).table("model_kwebs_queues").insert(task)
        key={"target":target,"args":args,"task":task}
        Queues.__scripttask.put(key)
        return taskid
    def getall(pagenow=1,pagesize=20,where=None):
        """获取全部队列

        code 1获取失败的任务   2获取等待中的任务   3获取正在执行中的任务  4获取执行完成的任务

        return list
        """
        if not os.path.isfile(queuesdbpath):
            model_kwebs_tasks=model_kwebs_task()
            model_kwebs_tasks.create_table()
        try:
            lists=kcsqlite.sqlite.connect(queuesdbpath,th_lock=True).where(where).table("model_kwebs_queues").order("id desc").page(pagenow,pagesize).select()
            count=kcsqlite.sqlite.connect(queuesdbpath,th_lock=True).where(where).table("model_kwebs_queues").count()
            return lists,count
        except:
            model_kwebs_tasks=model_kwebs_task()
            model_kwebs_tasks.create_table()
            return [],0
    def status(taskid):
        """获取任务状态
        
        taskid  任务id

        return dict
        """
        arr=kcsqlite.sqlite.connect(queuesdbpath,th_lock=True).table("model_kwebs_queues").where("taskid",taskid).find()
        if arr:
            arr['start']=float(arr['start'])
            dqsjc=arr['updtime']-arr['addtime']
            if dqsjc and arr['start']>=1:
                arr['starts']=round(float(arr['start'])/dqsjc,3)
            else:
                try:
                    arr['starts']=round(float(arr['starts']),3)
                except:
                    arr['starts']=0
        return arr
    def iscomplete():
        """判断对列中的任务是否全部执行完成
        
        return Boolean
        """
        if kcsqlite.sqlite.connect(queuesdbpath,th_lock=True).table("model_kwebs_queues").where("code","in",2,3).count():
            return True
        else:
            return False



from .common import *
class index:
    def index():
        response.title="模板标题"
        response.content="模板内容"
        return response.tpl()
    def home():
        return response.tpl()
    def api():
        return response.json()



# 打包上传 python setup.py sdist upload
# 安装 python setup.py install
import os,sys
from setuptools import setup
from kwebs import kwebsinfo
confkcws={}
confkcws['name']=kwebsinfo['name']                             #项目的名称 
confkcws['version']=kwebsinfo['version']							#项目版本
confkcws['description']=kwebsinfo['description']       #项目的简单描述
confkcws['long_description']=kwebsinfo['long_description']     #项目详细描述
confkcws['license']=kwebsinfo['license']                    #开源协议   mit开源
confkcws['url']=kwebsinfo['url']
confkcws['author']=kwebsinfo['author']  					 #名字
confkcws['author_email']=kwebsinfo['author_email'] 	     #邮件地址
confkcws['maintainer']=kwebsinfo['maintainer'] 						 #维护人员的名字
confkcws['maintainer_email']=kwebsinfo['maintainer_email']    #维护人员的邮件地址
def get_file(folder='./',lists=[]):
    lis=os.listdir(folder)
    for files in lis:
        if not os.path.isfile(folder+"/"+files):
            if files=='__pycache__' or files=='.git':
                pass
            else:
                lists.append(folder+"/"+files)
                get_file(folder+"/"+files,lists)
        else:
            pass
    return lists
def start():
    b=get_file("kwebs",['kwebs'])
    setup(
        name = confkcws["name"],
        version = confkcws["version"],
        keywords = "kwebs"+confkcws['version'],
        description = confkcws["description"],
        long_description = confkcws["long_description"],
        license = confkcws["license"],
        author = confkcws["author"],
        author_email = confkcws["author_email"],
        maintainer = confkcws["maintainer"],
        maintainer_email = confkcws["maintainer_email"],
        url=confkcws['url'],
        packages =  b,
        
        install_requires = ['kunapi>=1.1','kcmysql>=1','kcsqlite>=1','kchttp>=1','python-dateutil==2.9.0',
                            'pymongo==3.10.0','Mako==1.3.6','six>=1.12.0',
                            # 'websockets==10.4',
                            'curl_cffi==0.9.0',
                            'websockets==8.1',
                            ], #第三方包
        package_data = {
            '': ['*.html', '*.js','*.css','*.jpg','*.png','*.gif'],
        },
        entry_points = {
            'console_scripts':[
                'kwebs = kwebs.kwebs:cill_start'
            ]
        }
    )
start()